@@ -1,23 +1,23 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 2006-2010 Yves Renard, Julien Pommier.
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-//===========================================================================
+/*===========================================================================
+ 
+ Copyright (C) 2006-2013 Yves Renard, Julien Pommier.
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+===========================================================================*/
 
 #include <getfemint_misc.h>
 #include <getfem/getfem_derivatives.h>
@@ -144,7 +144,32 @@ gf_interpolate(getfemint::mexargs_in& in, getfemint::mexargs_out& out,
     garray<T> V = out.pop().create_array(dims, T());
     sl->interpolate(mf, U, V);
   }
-  else THROW_BADARG("expecting a mesh_fem or a mesh_slice for interpolation");
+  else {
+    size_type N = mf.linked_mesh().dim();
+    darray st = in.pop().to_darray();
+    std::vector<double> PTS(st.begin(), st.end());
+    size_type nbpoints = gmm::vect_size(PTS) / N;
+    getfem::base_node p(N);
+    getfem::mesh_trans_inv mti(mf.linked_mesh());
+    for (size_type i = 0; i < nbpoints; ++i) {
+      gmm::copy(gmm::sub_vector(PTS, gmm::sub_interval(i*N, N)), p);
+      // cout << "adding point" << p << endl;
+      mti.add_point(p);
+    }
+    
+    size_type qmult = mf.get_qdim();
+    if (qmult != 1) dims.push_back(unsigned(qmult));
+    dims.push_back(unsigned(nbpoints));
+    dims.opt_transform_col_vect_into_row_vect();
+    garray<T> V = out.pop().create_array(dims,T());
+
+    getfem::base_matrix Maux;
+    // cout << "begin interpolation, qmult = " << qmult << endl;
+    getfem::interpolation(mf, mti, U, V, Maux, 0);
+    // cout << "end interpolation" << endl;
+
+  }
+  // else THROW_BADARG("expecting a mesh_fem or a mesh_slice for interpolation");
 }
 
 bool U_is_a_vector(const rcarray &U, const std::string& cmd) {
@@ -220,6 +245,36 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        else out.pop().from_scalar(getfem::asm_L2_norm(*mim, *mf, U.cplx(),bv));
        );
 
+    /*@FUNC n = ('L2 dist', @tmim mim, @tmf mf2, @vec U2[, @mat CVids])
+    Compute the L2 distance between `U` and `U2`.
+
+    If `CVids` is given, the norm will be computed only on the listed
+    convexes.@*/
+    sub_command
+      ("L2 dist", 3, 4, 0, 1,
+       U_is_a_vector(U, "L2 dist");
+       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
+       const getfem::mesh_fem *mf_2 = in.pop().to_const_mesh_fem();
+       if (!U.is_complex()) {
+         darray st = in.pop().to_darray();
+         std::vector<double> V(st.begin(), st.end());
+	 dal::bit_vector bv = in.remaining() ?
+	   in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
+
+	 out.pop().from_scalar(getfem::asm_L2_dist(*mim, *mf, U.real(),
+						   *mf_2, V, bv));
+       } else {
+	 GMM_ASSERT1(false, "Sorry, complex version to be done");
+//          carray st = in.pop().to_carray();
+//          std::vector<std::complex<double> > V(st.begin(), st.end());
+// 	 dal::bit_vector bv = in.remaining() ?
+// 	   in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
+
+//          out.pop().from_scalar(getfem::asm_L2_dist(*mim, *mf, U.cplx(),
+// 						   *mf_2, V, bv));
+       }
+       );
+
 
    /*@FUNC n = ('H1 semi norm', @tmim mim[, @mat CVids])
     Compute the L2 norm of grad(`U`).
@@ -240,6 +295,37 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        );
 
 
+    /*@FUNC n = ('H1 semi dist', @tmim mim, @tmf mf2, @vec U2[, @mat CVids])
+    Compute the semi H1 distance between `U` and `U2`.
+
+    If `CVids` is given, the norm will be computed only on the listed
+    convexes.@*/
+    sub_command
+      ("H1 semi dist", 3, 4, 0, 1,
+       U_is_a_vector(U, "H1 semi dist");
+       const getfem::mesh_im *mim = in.pop().to_const_mesh_im();
+       const getfem::mesh_fem *mf_2 = in.pop().to_const_mesh_fem();
+       if (!U.is_complex()) {
+         darray st = in.pop().to_darray();
+         std::vector<double> V(st.begin(), st.end());
+	 dal::bit_vector bv = in.remaining() ?
+	   in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
+
+	 out.pop().from_scalar(getfem::asm_H1_semi_dist(*mim, *mf, U.real(),
+						   *mf_2, V, bv));
+       } else {
+	 GMM_ASSERT1(false, "Sorry, complex version to be done");
+//          carray st = in.pop().to_carray();
+//          std::vector<std::complex<double> > V(st.begin(), st.end());
+// 	 dal::bit_vector bv = in.remaining() ?
+// 	   in.pop().to_bit_vector(&mf->convex_index()) : mf->convex_index();
+
+//          out.pop().from_scalar(getfem::asm_L2_dist(*mim, *mf, U.cplx(),
+// 						   *mf_2, V, bv));
+       }
+       );
+
+
     /*@FUNC n = ('H1 norm', @tmim mim[, @mat CVids])
     Compute the H1 norm of `U`.
 
@@ -367,8 +453,8 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        );
 
 
-    /*@FUNC Ui = ('interpolate on', {@tmf mfi | @tsl sli})
-    Interpolate a field on another @tmf or a @tsl.
+    /*@FUNC Ui = ('interpolate on', {@tmf mfi | @tsl sli | @vec pts})
+    Interpolate a field on another @tmf or a @tsl or a list of points.
 
     - Interpolation on another @tmf `mfi`:
        `mfi` has to be Lagrangian. If `mf` and `mfi` share the same
@@ -378,6 +464,7 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
        mesh, but it is much faster. This can also be used with
        SLICE:INIT('points') to obtain field values at a given set of
        points.
+    - Interpolation on a set of points `pts`
 
     See also ::ASM('interpolation matrix')
     @*/
@@ -430,19 +517,55 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
 	 getfem::error_estimate(mim, *mf, U.cplx(), err, mim.convex_index());
        );
 
-    /*@FUNC E = ('convect', @tmf mf_v, @dvec V, @scalar dt, @int nt[, @str option])
+      
+#ifdef EXPERIMENTAL_PURPOSE_ONLY
+            
+    /*@FUNC E = ('error estimate nitsche', @tmim mim, @int GAMMAC, @int GAMMAN, @scalar lambda_, @scalar mu_, @scalar gamma0, @scalar f_coeff)
+    Compute an a posteriori error estimate in the case of Nitsche method.
+
+    Currently there is only one which is available: for each convex,
+    the jump of the normal derivative is integrated on its faces.@*/
+    sub_command
+      ("error_estimate_nitsche", 7, 7, 0, 1,
+       const getfem::mesh_im &mim = *in.pop().to_const_mesh_im();
+       int GAMMAC = in.pop().to_integer();
+       int GAMMAN = in.pop().to_integer();
+       scalar_type lambda = in.pop().to_scalar();
+       scalar_type mu = in.pop().to_scalar();
+       scalar_type gamma0 = in.pop().to_scalar();
+       scalar_type f_coeff = in.pop().to_scalar();
+       unsigned si = unsigned(mim.linked_mesh().convex_index().last_true()+1);
+       darray err =
+       out.pop().create_darray_h(si);
+       getfem::base_vector ERR(si);
+       getfem::base_vector UU(U.real().size());
+       gmm::copy(U.real(), UU);
+       getfem::error_estimate_nitsche(mim, *mf, UU, GAMMAC, GAMMAN, lambda, mu, gamma0, f_coeff, ERR);
+       gmm::copy(ERR, err);
+       );   
+      
+#endif
+      
+      
+      
+      
+    /*@FUNC E = ('convect', @tmf mf_v, @dvec V, @scalar dt, @int nt[, @str option[, @dvec per_min, @dvec per_max]])
     Compute a convection of `U` with regards to a steady state velocity
     field `V` with a Characteristic-Galerkin method. This
     method is restricted to pure Lagrange fems for U. `mf_v` should represent
     a continuous finite element method. `dt` is the integration time and `nt`
     is the number of integration step on the caracteristics. `option` is an
     option for the part of the boundary where there is a re-entrant convection.
-    `option = 'extrapolation'` for an extrapolation on the nearest element
-    or `option = 'unchanged'` for a constant value on that boundary.
+    `option = 'extrapolation'` for an extrapolation on the nearest element,
+    `option = 'unchanged'` for a constant value on that boundary or
+    `option = 'periodicity'` for a peridiodic boundary. For this latter option
+    the two vectors per_min, per_max has to be given and represent the limits
+    of the periodic domain (on components where per_max[k] < per_min[k]
+    no operation is done).
     This method is rather dissipative, but stable.
     @*/
     sub_command
-      ("convect", 4, 5, 0, 0,
+      ("convect", 4, 7, 0, 0,
        const getfem::mesh_fem *mf_v = in.pop().to_const_mesh_fem();
        rcarray V              = in.pop().to_rcarray();
        scalar_type dt = in.pop().to_scalar();
@@ -454,17 +577,35 @@ void gf_compute(getfemint::mexargs_in& m_in, getfemint::mexargs_out& m_out) {
 	 opt = getfem::CONVECT_EXTRAPOLATION;
        else if (cmd_strmatch(option, "extrapolation"))
 	 opt = getfem::CONVECT_EXTRAPOLATION;
+       else if (cmd_strmatch(option, "periodicity"))
+	 opt = getfem::CONVECT_PERIODICITY;
        else if (cmd_strmatch(option, "unchanged"))
 	 opt = getfem::CONVECT_UNCHANGED;
        else 
 	 THROW_BADARG("Bad option " << option<< " for convect command. "
-		      "should be 'extrapolation' or 'unchanged'");
+		      "should be 'extrapolation', 'unchanged' or "
+                      "'periodicity'");
+
+       getfem::base_node per_min;
+       getfem::base_node per_max;
+       if (in.remaining()) {
+         rcarray pmin = in.pop().to_rcarray();
+         rcarray pmax = in.pop().to_rcarray();
+         size_type N = mf_v->linked_mesh().dim();
+         per_min.resize(N);
+         per_max.resize(N); 
+         gmm::copy(pmin.real(), per_min);
+         gmm::copy(pmax.real(), per_max);
+       }
 
        if (U.is_complex() || V.is_complex())
 	 THROW_BADARG("Sorry, complex version of convect to be interfaced");
-       getfem::convect(*mf, U.real(), *mf_v, V.real(), dt, nt, opt);
+       getfem::convect(*mf, U.real(), *mf_v, V.real(),
+                       dt, nt, opt, per_min, per_max);
 
        );
+
+
   }
   
   
