@@ -1,6 +1,6 @@
 /*===========================================================================
  
- Copyright (C) 2004-2012 Julien Pommier.
+ Copyright (C) 2004-2014 Julien Pommier.
  
  This file is a part of GETFEM++
  
@@ -673,14 +673,14 @@ getfem_env(PyObject *self, PyObject *args) {
     word_out = PyString_FromString("GetFEM++");
   }else if (strcmp(word_in,"copyright") == 0){
     word_out = PyString_FromString
-    ("2004-2012 Yves Renard, Julien Pommier");
+    ("2004-2014 Yves Renard, Julien Pommier");
   }else if (strcmp(word_in,"authors") == 0){
     word_out = PyString_FromString
     ("Yves Renard, Julien Pommier");
   }else if (strcmp(word_in,"url") == 0){
     word_out = PyString_FromString("http://home.gna.org/getfem/");
   }else if (strcmp(word_in,"license") == 0){
-    word_out = PyString_FromString("GNU LGPL v2.1");
+    word_out = PyString_FromString("GNU LGPL v3");
   }else if (strcmp(word_in,"package") == 0){
     word_out = PyString_FromString(GETFEM_PACKAGE);
   }else if (strcmp(word_in,"package_name") == 0){
