@@ -1,32 +1,33 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 1999-2009 Yves Renard
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// As a special exception, you  may use  this file  as it is a part of a free
-// software  library  without  restriction.  Specifically,  if   other  files
-// instantiate  templates  or  use macros or inline functions from this file,
-// or  you compile this  file  and  link  it  with other files  to produce an
-// executable, this file  does  not  by itself cause the resulting executable
-// to be covered  by the GNU Lesser General Public License.  This   exception
-// does not  however  invalidate  any  other  reasons why the executable file
-// might be covered by the GNU Lesser General Public License.
-//
-//===========================================================================
+/* -*- c++ -*- (enables emacs c++ mode) */
+/*===========================================================================
+ 
+ Copyright (C) 1999-2012 Yves Renard
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+ As a special exception, you  may use  this file  as it is a part of a free
+ software  library  without  restriction.  Specifically,  if   other  files
+ instantiate  templates  or  use macros or inline functions from this file,
+ or  you compile this  file  and  link  it  with other files  to produce an
+ executable, this file  does  not  by itself cause the resulting executable
+ to be covered  by the GNU Lesser General Public License.  This   exception
+ does not  however  invalidate  any  other  reasons why the executable file
+ might be covered by the GNU Lesser General Public License.
+ 
+===========================================================================*/
 
 /**@file getfem_mesh.h
    @author  Yves Renard <Yves.Renard@insa-lyon.fr>
@@ -45,10 +46,19 @@
 #include "getfem_context.h"
 #include "getfem_mesh_region.h"
 
+#if defined(__GNUC__) 
+#if __cplusplus > 199711L
+#include <memory>  //on GCC need to check for C++ 11
+#endif
+#else
+#include <memory>  //all the others like Intel and MSVC have it
+#endif
+
+
 namespace getfem {
 
   /* Version counter for convexes. */
-  gmm::uint64_type act_counter(void);
+  gmm::uint64_type APIDECL act_counter(void);
 
   class integration_method;
   typedef boost::intrusive_ptr<const integration_method> pintegration_method;
@@ -90,9 +100,20 @@ namespace getfem {
       (same thing for the convexes, always use convex_index()).
   */
 
-  class mesh : virtual public dal::static_stored_object,
+  class APIDECL mesh : virtual public dal::static_stored_object,
 		       public bgeot::basic_mesh,
-		       public context_dependencies {
+		       public context_dependencies 
+
+//allowing mesh::shared_from_this() call
+#if defined(__GNUC__) 
+#if __cplusplus > 199711L
+           , public std::enable_shared_from_this<mesh>
+#endif
+#else
+           , public std::enable_shared_from_this<mesh>
+#endif
+  
+  {
   public :
 
     typedef bgeot::basic_mesh::PT_TAB PT_TAB;
@@ -110,8 +131,8 @@ namespace getfem {
      * When a new field is added, do NOT forget to add it in copy_from method!
      */
 
-    dal::dynamic_array<mesh_region> cvf_sets;
-    dal::bit_vector valid_cvf_sets;
+    mutable std::map<size_type, mesh_region> cvf_sets;
+    mutable dal::bit_vector valid_cvf_sets;
     void handle_region_refinement(size_type, const std::vector<size_type> &,
 				  bool);
 
@@ -123,7 +144,8 @@ namespace getfem {
 #if GETFEM_PARA_LEVEL > 1
     mutable bool modified;
     mutable mesh_region mpi_region;
-    mutable dal::dynamic_array<mesh_region> mpi_sub_region;
+    mutable std::map<size_type, mesh_region> mpi_sub_region;
+    // mutable dal::dynamic_array<mesh_region> mpi_sub_region;
     mutable dal::bit_vector valid_sub_regions;
 
     void touch(void) {
@@ -158,8 +180,9 @@ namespace getfem {
 #endif
 
     /// Constructor.
-    mesh(void);
-    mesh(const bgeot::basic_mesh &m);
+    explicit mesh(const std::string name = "");
+    mesh(const bgeot::basic_mesh &m, const std::string name = "");
+    inline std::string get_name() const {return name_;}
     void update_from_context(void) const {}
     /// Mesh dimension.
     dim_type dim(void) const { return pts.dim(); }
@@ -330,6 +353,12 @@ namespace getfem {
     template<class ITER>
     size_type add_prism_by_points(dim_type di, const ITER &ps);
 
+    size_type add_face_of_convex(size_type, short_type)
+    { GMM_ASSERT1(false, "Sorry, to be done"); }
+
+    void add_faces_of_convex(size_type)
+    { GMM_ASSERT1(false, "Sorry, to be done"); }
+
     /// Delete the convex of index ic from the mesh.
     void sup_convex(size_type ic, bool sup_points = false);
     /** Swap the indexes of the convex of indexes i and j
@@ -359,6 +388,17 @@ namespace getfem {
     */
     base_small_vector normal_of_face_of_convex(size_type ic, short_type f,
 					       size_type n=0) const;
+
+
+    /* Return the mean outward unit normal vector of the given convex
+       face computed on the geometrical nodes of the face.
+
+       @param ic the convex number.
+       @param f the face number.
+       @return the face normal.
+    */
+    base_small_vector mean_normal_of_face_of_convex(size_type ic,
+                                                    short_type f) const;
     /** Return a local basis for the convex face.
        @param ic the convex number.
        @param f the face number.
@@ -389,6 +429,8 @@ namespace getfem {
     scalar_type convex_radius_estimate(size_type ic) const;
     /** Return an estimate of the convex smallest dimension. @see getfem::convex_radius_estimate */
     scalar_type minimal_convex_radius_estimate() const;
+    /** Return an estimate of the convex largest dimension. @see getfem::convex_radius_estimate */
+    scalar_type maximal_convex_radius_estimate() const;
     /** Apply the given translation to each mesh node. */
     void translation(const base_small_vector &);
     /** apply the given matrix transformation to each mesh node. */
@@ -396,21 +438,25 @@ namespace getfem {
     /** Return the bounding box [Pmin - Pmax] of the mesh. */
     void bounding_box(base_node& Pmin, base_node &Pmax) const;
     /** Return the region of index 'id'. Regions stored in mesh are
-	automagically created on their first use. Moreover, they are
 	updated when the mesh is modified (i.e. when a convex is
 	removed from the mesh, it is also removed from all regions of
 	the mesh.
     */
     const mesh_region region(size_type id) const {
-      if (has_region(id)) return cvf_sets[id];
-      else return mesh_region(const_cast<mesh&>(*this),id);
+      if (id == mesh_region::all_convexes().id())
+        return mesh_region::all_convexes();
+      else if (!has_region(id)) {
+        valid_cvf_sets.add(id);
+        cvf_sets[id] = mesh_region(const_cast<mesh&>(*this),id);
+      }
+      return cvf_sets[id];
     }
     /* Return a reference such that operator= works as expected */
     mesh_region &region(size_type id) {
-      if (!has_region(id))
-	/* will be added into valid_cvf_sets
-	   later via mesh_region::maybe_notify_parent_mesh */
-	cvf_sets[id] = mesh_region(*this,id);
+      if (!has_region(id)) {
+        valid_cvf_sets.add(id);
+        cvf_sets[id] = mesh_region(*this,id);
+      }
       return cvf_sets[id];
     }
     /** Return true if the region of index 's' exists in the mesh */
@@ -464,7 +510,7 @@ namespace getfem {
     friend class mesh_region;
   private:
     void swap_convex_in_regions(size_type c1, size_type c2);
-    void touch_from_region(size_type id) { valid_cvf_sets.add(id); touch(); }
+    void touch_from_region(size_type /*id*/) { touch(); }
     void to_edges() {} /* to be done, the to_edges of mesh_structure does   */
                        /* not handle geotrans */
 
@@ -498,6 +544,9 @@ namespace getfem {
 
     Bank_info_struct *Bank_info;
 
+    std::string name_; //optional name of the mesh
+    void set_name(const std::string&);
+
     void Bank_convex_with_edge(size_type, size_type,
 			       std::vector<size_type> &);
     bool Bank_is_convex_having_points(size_type,
@@ -518,14 +567,14 @@ namespace getfem {
 
   };
 
-  inline void mesh::add_face_to_set(size_type s, size_type c, short_type f) {
+  inline void APIDECL mesh::add_face_to_set(size_type s, size_type c, short_type f) {
     region(s).add(c,f);
   }
 
   /**
    * build a N+1 dimensions mesh from a N-dimensions mesh by extrusion.
    */
-  void extrude(const mesh& in, mesh& out, unsigned nb_layers);
+  void APIDECL extrude(const mesh& in, mesh& out, unsigned nb_layers);
 
  template<class ITER>
     size_type mesh::add_convex_by_points(bgeot::pgeometric_trans pgt,
@@ -567,24 +616,24 @@ namespace getfem {
       @param pai the approximate integration used for the computation
       of the convex area.
    */
-  scalar_type convex_area_estimate(bgeot::pgeometric_trans pgt,
+  scalar_type APIDECL convex_area_estimate(bgeot::pgeometric_trans pgt,
 				   const base_matrix& pts,
 				   pintegration_method pim);
 
   /** rough estimate of the maximum value of the condition
    * number of the jacobian of the geometric transformation */
-  scalar_type convex_quality_estimate(bgeot::pgeometric_trans pgt,
+  scalar_type APIDECL convex_quality_estimate(bgeot::pgeometric_trans pgt,
 				      const base_matrix& pts);
 
   /** rough estimate of the radius of the convex using the largest eigenvalue
    * of the jacobian of the geometric transformation */
-  scalar_type convex_radius_estimate(bgeot::pgeometric_trans pgt,
+  scalar_type APIDECL convex_radius_estimate(bgeot::pgeometric_trans pgt,
 				     const base_matrix& pts);
 
   /* stores a convex face. if f == -1, it is the whole convex.             */
   struct convex_face;
   typedef std::vector<convex_face> convex_face_ct;
-  struct convex_face  {
+  struct APIDECL convex_face  {
     size_type cv;
     size_type f;
     inline bool operator < (const convex_face &e) const
@@ -602,19 +651,32 @@ namespace getfem {
    * (i.e. faces which are not shared by two convexes)
    * + convexes whose dimension is smaller that m.dim()
    */
-  void  outer_faces_of_mesh(const mesh &m, const dal::bit_vector& cvlst,
+  void  APIDECL outer_faces_of_mesh(const mesh &m, const dal::bit_vector& cvlst,
 			    convex_face_ct& flist);
 
-  inline void outer_faces_of_mesh(const mesh &m, convex_face_ct& flist)
+  inline void APIDECL outer_faces_of_mesh(const mesh &m, convex_face_ct& flist)
     IS_DEPRECATED;
   inline void outer_faces_of_mesh(const mesh &m, convex_face_ct& flist)
   { outer_faces_of_mesh(m,m.convex_index(),flist); }
 
-  void  outer_faces_of_mesh(const mesh &m, const mesh_region &cvlst,
+  void  APIDECL outer_faces_of_mesh(const mesh &m, const mesh_region &cvlst,
 			    mesh_region &flist);
 
-  inline void  outer_faces_of_mesh(const mesh &m, mesh_region &flist)
+  inline void  APIDECL outer_faces_of_mesh(const mesh &m, mesh_region &flist)
   { outer_faces_of_mesh(m,m.convex_index(),flist); }
+
+
+  inline mesh_region APIDECL outer_faces_of_mesh(const mesh &m)
+  { mesh_region fl; outer_faces_of_mesh(m,m.convex_index(),fl); return fl; }
+
+  /** Select in the region mr the faces of the mesh m with their unit
+      outward vector having a maximal angle "angle" with the vector V.
+   */ 
+  mesh_region APIDECL select_faces_of_normal(const mesh &m, const mesh_region &mr,
+                                     const base_small_vector &V,
+                                     scalar_type angle);
+
+
   ///@}
 }  /* end of namespace getfem.                                             */
 
