@@ -1,23 +1,23 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 1999-2008 Yves Renard
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-//===========================================================================
+/*===========================================================================
+ 
+ Copyright (C) 1999-2012 Yves Renard
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+===========================================================================*/
 
 /** @file getfem_fem.cc
     @author Yves Renard <Yves.Renard@insa-lyon.fr>
@@ -30,12 +30,13 @@
 #include "gmm/gmm_algobase.h"
 #include "getfem/dal_naming_system.h"
 #include "getfem/getfem_fem.h"
-#include "getfem/getfem_gauss_lobatto_fem_coef.h"
-#include "getfem/getfem_integration.h" /* for gauss-lobatto points */
+#include "getfem/getfem_gauss_lobatto_fem_coef.h" /* for gauss-lobatto points*/
+#include "getfem/getfem_integration.h"
+#include "getfem/getfem_omp.h"
+#include "getfem/getfem_torus.h"
 
 namespace getfem {
 
-  typedef dal::naming_system<virtual_fem>::param_list fem_param_list;
 
   const base_matrix& fem_interpolation_context::M() const {
     if (gmm::mat_nrows(M_) == 0) {
@@ -51,11 +52,20 @@ namespace getfem {
     return convex_num_; 
   }
 
+  bool fem_interpolation_context::is_convex_num_valid() const {
+    return convex_num_ != size_type(-1); 
+  }
+
   size_type fem_interpolation_context::face_num() const {
-    GMM_ASSERT3(face_num_ != size_type(-1), "");
+    GMM_ASSERT3(face_num_ != size_type(-1),
+		"Face number is asked but not defined");
     return face_num_; 
   }
 
+  bool fem_interpolation_context::is_on_face() const {
+    return (face_num_ != size_type(-1));
+  }
+
   void fem_interpolation_context::base_value(base_tensor& t,
 					     bool withM) const {
     if (pf()->is_on_real_element())
@@ -267,8 +277,8 @@ namespace getfem {
   typedef dal::dynamic_tree_sorted<dof_description, dof_description_comp__> dof_d_tab;
 
   pdof_description lagrange_dof(dim_type n) {
-    static dim_type n_old = dim_type(-2);
-    static pdof_description p_old = 0;
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(dim_type, n_old, dim_type(-2));
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pdof_description, p_old, 0);
     if (n != n_old) {
       dof_d_tab& tab = dal::singleton<dof_d_tab>::instance();
       dof_description l;
@@ -281,8 +291,8 @@ namespace getfem {
   } 
 
   pdof_description lagrange_0_dof(dim_type n) {
-    static dim_type n_old = dim_type(-2);
-    static pdof_description p_old = 0;
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(dim_type, n_old, dim_type(-2));
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pdof_description, p_old, 0);
     if (n != n_old) {
       dof_d_tab& tab = dal::singleton<dof_d_tab>::instance();
       dof_description l;
@@ -557,7 +567,7 @@ namespace getfem {
       
       w[0] = K;
       for (short_type nn = 1; nn <= N; ++nn) { 
-	w[nn]=short_type(floor(0.5+((cv_node.points()[i])[nn-1]*opt_long_scalar_type(K))));
+	w[nn]=short_type(floor(0.5+bgeot::to_scalar((cv_node.points()[i])[nn-1]*opt_long_scalar_type(K))));
 	w[0]=short_type(w[0] - w[nn]);
       }
       
@@ -917,9 +927,6 @@ namespace getfem {
     return fem_descriptor(name.str());
   }
 
-  static void read_poly(bgeot::base_poly &p, int d, const char *s) 
-  { p = bgeot::read_base_poly(short_type(d), s); }
-
   /* ******************************************************************** */
   /*	P1 NON CONFORMING (dim 2)                                         */
   /* ******************************************************************** */
@@ -949,48 +956,121 @@ namespace getfem {
 
 
   /* ******************************************************************** */
-  /*	Quad8 SERENDIPITY ELEMENT (dim 2) (incomplete Q2)                 */
+  /*	Quad8/Hexa20 SERENDIPITY ELEMENT (dim 2 or 3) (incomplete Q2)     */
   /* ******************************************************************** */
 
-  // local dof numeration:
-  // 6--5--4 
+  // local dof numeration for 2D:
+  // 5--6--7
   // |     |
-  // 7     3
+  // 3     4
   // |     |
   // 0--1--2
+  //
+  // local dof numeration for 3D:
+  //
+  //       17---18---19
+  //      /|        /|
+  //     / 10      / 11
+  //   15  |      16 |
+  //   /   5----6/---7
+  //  /   /     /   /
+  // 12---13---14  /
+  // |  3      |  4
+  // 8 /       9 /
+  // |/        |/
+  // 0----1----2
 
-   static pfem incomplete_Q2_fem(fem_param_list &params,
+   static pfem Q2_incomplete_fem(fem_param_list &params,
 	std::vector<dal::pstatic_stored_object> &dependencies) {
-    GMM_ASSERT1(params.size() == 0, "Bad number of parameters");
+    GMM_ASSERT1(params.size() <= 1, "Bad number of parameters");
+    dim_type n = 2;
+    if (params.size() > 0) {
+      GMM_ASSERT1(params[0].type() == 0, "Bad type of parameters");
+      n = dim_type(::floor(params[0].num() + 0.01));
+       GMM_ASSERT1(n == 2 || n == 3, "Bad parameter, expected value 2 or 3");
+    }
     fem<base_poly> *p = new fem<base_poly>;
-    p->mref_convex() = bgeot::parallelepiped_of_reference(2);
-    p->dim() = 2;
+    p->mref_convex() = bgeot::parallelepiped_of_reference(n);
+    p->dim() = n;
     p->is_equivalent() = p->is_polynomial() = p->is_lagrange() = true;
     p->estimated_degree() = 2;
     p->init_cvs_node();
-    p->base().resize(8);
+    p->base().resize(n == 2 ? 8 : 20);
 
-    std::stringstream s
-      ( "1 - 2*x^2*y - 2*x*y^2 + 2*x^2 + 5*x*y + 2*y^2 - 3*x - 3*y;"
-	"4*(x^2*y - x^2 - x*y + x);"
-	"2*x*y*y - 2*x*x*y + 2*x*x - x*y - x;"
-	"4*(x*y - x*y*y);"
-	"2*x*x*y + 2*x*y*y - 3*x*y;"
-	"4*(x*y - x*x*y);"
-	"2*x*x*y - 2*x*y*y - x*y + 2*y*y - y;"
-	"4*(x*y*y - x*y - y*y + y);");
+    if (n == 2) {
+      std::stringstream s
+        ( "1 - 2*x^2*y - 2*x*y^2 + 2*x^2 + 5*x*y + 2*y^2 - 3*x - 3*y;"
+          "4*(x^2*y - x^2 - x*y + x);"
+          "2*x*y*y - 2*x*x*y + 2*x*x - x*y - x;"
+          "4*(x*y*y - x*y - y*y + y);"
+          "4*(x*y - x*y*y);"
+          "2*x*x*y - 2*x*y*y - x*y + 2*y*y - y;"
+          "4*(x*y - x*x*y);"
+          "2*x*x*y + 2*x*y*y - 3*x*y;");
 
-    for (int i = 0; i < 8; ++i) p->base()[i] = bgeot::read_base_poly(2, s);
+      for (int i = 0; i < 8; ++i) p->base()[i] = bgeot::read_base_poly(2, s);
 
-    p->add_node(lagrange_dof(2), base_small_vector(0.0, 0.0));
-    p->add_node(lagrange_dof(2), base_small_vector(0.5, 0.0));
-    p->add_node(lagrange_dof(2), base_small_vector(1.0, 0.0));
-    p->add_node(lagrange_dof(2), base_small_vector(1.0, 0.5));
-    p->add_node(lagrange_dof(2), base_small_vector(1.0, 1.0));
-    p->add_node(lagrange_dof(2), base_small_vector(0.5, 1.0));
-    p->add_node(lagrange_dof(2), base_small_vector(0.0, 1.0));
-    p->add_node(lagrange_dof(2), base_small_vector(0.0, 0.5));
-   
+      p->add_node(lagrange_dof(2), base_small_vector(0.0, 0.0));
+      p->add_node(lagrange_dof(2), base_small_vector(0.5, 0.0));
+      p->add_node(lagrange_dof(2), base_small_vector(1.0, 0.0));
+      p->add_node(lagrange_dof(2), base_small_vector(0.0, 0.5));
+      p->add_node(lagrange_dof(2), base_small_vector(1.0, 0.5));
+      p->add_node(lagrange_dof(2), base_small_vector(0.0, 1.0));
+      p->add_node(lagrange_dof(2), base_small_vector(0.5, 1.0));
+      p->add_node(lagrange_dof(2), base_small_vector(1.0, 1.0));
+    } else {
+      std::stringstream s
+        ("1 + 2*x^2*y*z + 2*x*y^2*z + 2*x*y*z^2"
+           " - 2*x^2*y - 2*x^2*z - 2*x*y^2 - 2*y^2*z - 2*y*z^2 - 2*x*z^2 - 7*x*y*z"
+           " + 2*x^2 + 2*y^2 + 2*z^2 + 5*y*z + 5*x*z + 5*x*y - 3*x - 3*y - 3*z;"
+         "4*( - x^2*y*z + x*y*z + x^2*z - x*z + x^2*y - x*y - x^2 + x);"
+         "2*x^2*y*z - 2*x*y^2*z - 2*x*y*z^2"
+           " - 2*x^2*y - 2*x^2*z + 2*x*y^2 + 2*x*z^2 + 3*x*y*z + 2*x^2 - x*y - x*z - x;"
+         "4*( - x*y^2*z + x*y^2 + y^2*z + x*y*z - x*y - y^2 - y*z + y);"
+         "4*(x*y^2*z - x*y^2 - x*y*z + x*y);"
+         " - 2*x^2*y*z + 2*x*y^2*z - 2*x*y*z^2"
+           " + 2*x^2*y - 2*x*y^2 - 2*y^2*z + 2*y*z^2 + 3*x*y*z - x*y + 2*y^2 - y*z - y;"
+         "4*(x^2*y*z - x^2*y - x*y*z + x*y);"
+         " - 2*x^2*y*z - 2*x*y^2*z + 2*x*y*z^2 + 2*x^2*y + 2*x*y^2 + x*y*z - 3*x*y;"
+         "4*( - x*y*z^2 + x*z^2 + y*z^2 + x*y*z - x*z - y*z - z^2 + z);"
+         "4*(x*y*z^2 - x*y*z - x*z^2 + x*z);"
+         "4*(x*y*z^2 - x*y*z - y*z^2 + y*z);"
+         "4*( - x*y*z^2 + x*y*z);"
+         " - 2*x^2*y*z - 2*x*y^2*z + 2*x*y*z^2"
+           " + 2*x^2*z + 2*y^2*z - 2*x*z^2 - 2*y*z^2 + 3*x*y*z - x*z - y*z + 2*z^2 - z;"
+         "4*(x^2*y*z - x^2*z - x*y*z + x*z);"
+         " - 2*x^2*y*z + 2*x*y^2*z - 2*x*y*z^2 + 2*x^2*z + 2*x*z^2 + x*y*z - 3*x*z;"
+         "4*(x*y^2*z - y^2*z - x*y*z + y*z);"
+         "4*( - x*y^2*z + x*y*z);"
+         "2*x^2*y*z - 2*x*y^2*z - 2*x*y*z^2 + 2*y^2*z + 2*y*z^2 + x*y*z - 3*y*z;"
+         "4*( - x^2*y*z + x*y*z);"
+         "2*x^2*y*z + 2*x*y^2*z + 2*x*y*z^2 - 5*x*y*z;");
+
+      for (int i = 0; i < 20; ++i) p->base()[i] = bgeot::read_base_poly(3, s);
+
+      p->add_node(lagrange_dof(3), base_small_vector(0.0, 0.0, 0.0));
+      p->add_node(lagrange_dof(3), base_small_vector(0.5, 0.0, 0.0));
+      p->add_node(lagrange_dof(3), base_small_vector(1.0, 0.0, 0.0));
+      p->add_node(lagrange_dof(3), base_small_vector(0.0, 0.5, 0.0));
+      p->add_node(lagrange_dof(3), base_small_vector(1.0, 0.5, 0.0));
+      p->add_node(lagrange_dof(3), base_small_vector(0.0, 1.0, 0.0));
+      p->add_node(lagrange_dof(3), base_small_vector(0.5, 1.0, 0.0));
+      p->add_node(lagrange_dof(3), base_small_vector(1.0, 1.0, 0.0));
+
+      p->add_node(lagrange_dof(3), base_small_vector(0.0, 0.0, 0.5));
+      p->add_node(lagrange_dof(3), base_small_vector(1.0, 0.0, 0.5));
+      p->add_node(lagrange_dof(3), base_small_vector(0.0, 1.0, 0.5));
+      p->add_node(lagrange_dof(3), base_small_vector(1.0, 1.0, 0.5));
+
+      p->add_node(lagrange_dof(3), base_small_vector(0.0, 0.0, 1.0));
+      p->add_node(lagrange_dof(3), base_small_vector(0.5, 0.0, 1.0));
+      p->add_node(lagrange_dof(3), base_small_vector(1.0, 0.0, 1.0));
+      p->add_node(lagrange_dof(3), base_small_vector(0.0, 0.5, 1.0));
+      p->add_node(lagrange_dof(3), base_small_vector(1.0, 0.5, 1.0));
+      p->add_node(lagrange_dof(3), base_small_vector(0.0, 1.0, 1.0));
+      p->add_node(lagrange_dof(3), base_small_vector(0.5, 1.0, 1.0));
+      p->add_node(lagrange_dof(3), base_small_vector(1.0, 1.0, 1.0));
+    }
     dependencies.push_back(p->ref_convex(0));
     dependencies.push_back(p->node_tab(0));
 
@@ -1393,8 +1473,14 @@ namespace getfem {
     init_cvs_node();
     std::stringstream sstr; sstr << "IM_GAUSSLOBATTO1D(" << k*2-1 << ")";
     pintegration_method gl_im = int_method_descriptor(sstr.str());
+    std::vector<base_node> points(k+1);
     for (size_type i = 0; i < k+1; ++i) {
-      add_node(lagrange_dof(1), gl_im->approx_method()->point(i));
+      points[i] = gl_im->approx_method()->point(i); 
+    }
+    std::sort(points.begin(),points.end());
+    for (size_type i = 0; i < k+1; ++i) {
+      // cout << points[i][0] << endl;
+      add_node(lagrange_dof(1), points[i]);
     }
     base_.resize(k+1);
     const double *coefs = fem_coeff_gausslob[k];
@@ -1430,10 +1516,10 @@ namespace getfem {
   void hermite_segment__::mat_trans(base_matrix &M,
 				    const base_matrix &G,
 				    bgeot::pgeometric_trans pgt) const {
-    static bgeot::pgeotrans_precomp pgp;
-    static bgeot::pgeometric_trans pgt_stored = 0;
-    static base_matrix K(1, 1);
-    static base_vector r(1);
+    DEFINE_STATIC_THREAD_LOCAL(bgeot::pgeotrans_precomp, pgp);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(bgeot::pgeometric_trans, pgt_stored, 0);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix, K, 1, 1);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_vector,r, 1);
     dim_type N = dim_type(G.nrows());
 
     if (pgt != pgt_stored) {
@@ -1453,7 +1539,7 @@ namespace getfem {
     if (!(pgt->is_linear())) gmm::mult(G, pgp->grad(3), K);
     if (N == 1) M(3, 3) = K(0,0);
     else M(3, 3) = gmm::mat_euclidean_norm(K)
-      * gmm::sgn(gmm::vect_sp(gmm::mat_row(K, 0), r));
+      * gmm::sgn(gmm::vect_sp(gmm::mat_col(K, 0), r));
   }
 
   // Hermite element on the segment. when the real element lies in
@@ -1472,13 +1558,13 @@ namespace getfem {
     pt[0] = 0.0; add_node(lagrange_dof(1), pt);
     read_poly(base_[0], 1, "(1 - x)^2*(2*x + 1)");
 
-    pt[0] = 0.0; add_node(derivative_dof(1, 0), pt, dal::bit_vector());
+    pt[0] = 0.0; add_node(derivative_dof(1, 0), pt);
     read_poly(base_[1], 1, "x*(x - 1)*(x - 1)");
 
     pt[0] = 1.0; add_node(lagrange_dof(1), pt);
     read_poly(base_[2], 1, "x*x*(3  - 2*x)");
 
-    pt[0] = 1.0; add_node(derivative_dof(1, 0), pt, dal::bit_vector());
+    pt[0] = 1.0; add_node(derivative_dof(1, 0), pt);
     read_poly(base_[3], 1, "x*x*(x - 1)");
   }
 
@@ -1495,9 +1581,10 @@ namespace getfem {
   void hermite_triangle__::mat_trans(base_matrix &M,
 				    const base_matrix &G,
 				    bgeot::pgeometric_trans pgt) const {
-    static bgeot::pgeotrans_precomp pgp;
-    static bgeot::pgeometric_trans pgt_stored = 0;
-    static base_matrix K(2, 2);
+
+    DEFINE_STATIC_THREAD_LOCAL(bgeot::pgeotrans_precomp, pgp);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(bgeot::pgeometric_trans, pgt_stored, 0);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix, K, 2, 2);
     dim_type N = dim_type(G.nrows());
 
     GMM_ASSERT1(N == 2, "Sorry, this version of hermite "
@@ -1567,9 +1654,9 @@ namespace getfem {
   void hermite_tetrahedron__::mat_trans(base_matrix &M,
 				    const base_matrix &G,
 				    bgeot::pgeometric_trans pgt) const {
-    static bgeot::pgeotrans_precomp pgp;
-    static bgeot::pgeometric_trans pgt_stored = 0;
-    static base_matrix K(3, 3);
+    DEFINE_STATIC_THREAD_LOCAL(bgeot::pgeotrans_precomp, pgp);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(bgeot::pgeometric_trans, pgt_stored, 0);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix, K, 3, 3);
     dim_type N = dim_type(G.nrows());
     GMM_ASSERT1(N == 3, "Sorry, this version of hermite "
 		"element works only on dimension three.")
@@ -1670,10 +1757,11 @@ namespace getfem {
   void argyris_triangle__::mat_trans(base_matrix &M,
 				    const base_matrix &G,
 				    bgeot::pgeometric_trans pgt) const {
-    static bgeot::pgeotrans_precomp pgp;
-    static pfem_precomp pfp;
-    static bgeot::pgeometric_trans pgt_stored = 0;
-    static base_matrix K(2, 2);
+    
+    DEFINE_STATIC_THREAD_LOCAL(bgeot::pgeotrans_precomp, pgp);
+    DEFINE_STATIC_THREAD_LOCAL(pfem_precomp, pfp);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(bgeot::pgeometric_trans, pgt_stored, 0);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix, K, 2, 2);
     dim_type N = dim_type(G.nrows());
     GMM_ASSERT1(N == 2, "Sorry, this version of argyris "
 		"element works only on dimension two.")
@@ -1712,7 +1800,7 @@ namespace getfem {
       M(5+6*k, 3+6*k) = c*c;     M(5+6*k, 4+6*k) = c*d;       M(5+6*k, 5+6*k) = d*d; 
     }
     
-    static base_matrix W(3, 21);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix, W, 3, 21);
     base_small_vector norient(M_PI, M_PI * M_PI);
     if (pgt->is_linear()) gmm::lu_inverse(K); 
     for (unsigned i = 18; i < 21; ++i) {
@@ -1731,10 +1819,11 @@ namespace getfem {
       for (unsigned j = 0; j < 21; ++j)
 	W(i-18, j) = t(j, 0, 0) * v[0] + t(j, 0, 1) * v[1];
     }
-    
-    static base_matrix A(3, 3);
-    static bgeot::base_vector w(3), coeff(3);
-    static gmm::sub_interval SUBI(18, 3), SUBJ(0, 3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix,A,3,3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(bgeot::base_vector, w, 3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(bgeot::base_vector, coeff, 3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(gmm::sub_interval, SUBI, 18,3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(gmm::sub_interval, SUBJ, 0,3);
     gmm::copy(gmm::sub_matrix(W, SUBJ, SUBI), A);
     gmm::lu_inverse(A);
     gmm::copy(gmm::transposed(A), gmm::sub_matrix(M, SUBI));
@@ -1839,10 +1928,11 @@ namespace getfem {
   void morley_triangle__::mat_trans(base_matrix &M,
 				    const base_matrix &G,
 				    bgeot::pgeometric_trans pgt) const {
-    static bgeot::pgeotrans_precomp pgp;
-    static pfem_precomp pfp;
-    static bgeot::pgeometric_trans pgt_stored = 0;
-    static base_matrix K(2, 2);
+
+    DEFINE_STATIC_THREAD_LOCAL(bgeot::pgeotrans_precomp, pgp);
+    DEFINE_STATIC_THREAD_LOCAL(pfem_precomp, pfp);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(bgeot::pgeometric_trans, pgt_stored, 0);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix, K, 2, 2);
     dim_type N = dim_type(G.nrows());
     GMM_ASSERT1(N == 2, "Sorry, this version of morley "
 		"element works only on dimension two.")
@@ -1853,8 +1943,7 @@ namespace getfem {
       pfp = fem_precomp(this, node_tab(0), 0);
     }
     gmm::copy(gmm::identity_matrix(), M);
-    
-    static base_matrix W(3, 6);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix, W, 3, 6);
     base_small_vector norient(M_PI, M_PI * M_PI);
     if (pgt->is_linear())
       { gmm::mult(G, pgp->grad(0), K); gmm::lu_inverse(K); }
@@ -1876,9 +1965,11 @@ namespace getfem {
     }
     //    cout << "W = " << W << endl; getchar();
     
-    static base_matrix A(3, 3);
-    static bgeot::base_vector w(3), coeff(3);
-    static gmm::sub_interval SUBI(3, 3), SUBJ(0, 3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_matrix, A, 3, 3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_vector, w, 3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(base_vector, coeff, 3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(gmm::sub_interval, SUBI, 3, 3);
+    DEFINE_STATIC_THREAD_LOCAL_CONSTRUCTED(gmm::sub_interval, SUBJ, 0, 3);
     gmm::copy(gmm::sub_matrix(W, SUBJ, SUBI), A);
     gmm::lu_inverse(A);
     gmm::copy(gmm::transposed(A), gmm::sub_matrix(M, SUBI));
@@ -1930,12 +2021,12 @@ namespace getfem {
   struct PK_discont_ : public PK_fem_ {
   public :
     
-    PK_discont_(dim_type nc, short_type k, scalar_type alpha=0.)
+    PK_discont_(dim_type nc, short_type k, scalar_type alpha=scalar_type(0))
       : PK_fem_(nc, k) {
       std::fill(dof_types_.begin(), dof_types_.end(),
 		lagrange_nonconforming_dof(nc));
 
-      if (alpha != 0.) {
+      if (alpha != scalar_type(0)) {
 	base_node G = 
 	  gmm::mean_value(cv_node.points().begin(), cv_node.points().end());
       for (size_type i=0; i < cv_node.nb_points(); ++i) 
@@ -2027,19 +2118,24 @@ namespace getfem {
   static pfem classical_fem_(const char *suffix, const char *arg, 
 			     bgeot::pgeometric_trans pgt,
 			     short_type k) {
-    static bgeot::pgeometric_trans pgt_last = 0;
-    static short_type k_last = short_type(-1);
-    static pfem fm_last = 0;
-    bool found = false;
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(bgeot::pgeometric_trans, pgt_last, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(short_type, k_last, short_type(-1));
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pfem, fm_last, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(char, isuffix_last, 0);
+    bool found = false, isuffix = suffix[0];
 
-    if (pgt_last == pgt && k_last == k)
+    if (pgt_last == pgt && k_last == k && isuffix == isuffix_last)
       return fm_last;
 
+    isuffix_last = isuffix;
 
     size_type n = pgt->structure()->dim();
     size_type nbp = pgt->basic_structure()->nb_points();
     std::stringstream name;
 
+    // Identifying if it is a torus structure
+    if(bgeot::is_torus_geom_trans(pgt) && n == 3) n = 2;
+
     /* Identifying P1-simplexes.                                          */
     if (nbp == n+1)
       if (pgt->basic_structure() == bgeot::simplex_structure(dim_type(n)))
@@ -2126,7 +2222,7 @@ namespace getfem {
       add_suffix("PK_FULL_HIERARCHICAL_COMPOSITE",
 		 PK_composite_full_hierarch_fem);
       add_suffix("PK_GAUSSLOBATTO1D", PK_GL_fem);
-      add_suffix("INCOMPLETE_Q2", incomplete_Q2_fem);
+      add_suffix("Q2_INCOMPLETE", Q2_incomplete_fem);
       add_suffix("HCT_TRIANGLE", HCT_triangle_fem);
       add_suffix("REDUCED_HCT_TRIANGLE", reduced_HCT_triangle_fem);
       add_suffix("QUADC1_COMPOSITE", quadc1p3_fem);
@@ -2152,14 +2248,20 @@ namespace getfem {
       shorter_name_of_method(p);
   }
 
+  // allows the add of a fem.
+  void add_fem_name(std::string name,
+		    dal::naming_system<virtual_fem>::pfunction f) {
+    dal::singleton<fem_naming_system>::instance().add_suffix(name, f);
+  }
+
   /* ******************************************************************** */
   /*    Aliases functions                                                 */
   /* ******************************************************************** */
 
   pfem PK_fem(size_type n, short_type k) {
-    static pfem pf = 0;
-    static size_type d = size_type(-2);
-    static short_type r = short_type(-2);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pfem, pf, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(size_type, d, size_type(-2));
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(short_type, r, short_type(-2));
     if (d != n || r != k) {
       std::stringstream name;
       name << "FEM_PK(" << n << "," << k << ")";
@@ -2170,9 +2272,9 @@ namespace getfem {
   }
 
   pfem QK_fem(size_type n, short_type k) {
-    static pfem pf = 0;
-    static size_type d = size_type(-2);
-    static short_type r = short_type(-2);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pfem, pf, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(size_type, d, size_type(-2));
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(short_type, r, short_type(-2));
     if (d != n || r != k) {
       std::stringstream name;
       name << "FEM_QK(" << n << "," << k << ")";
@@ -2183,9 +2285,9 @@ namespace getfem {
   }
 
   pfem PK_prism_fem(size_type n, short_type k) {
-    static pfem pf = 0;
-    static size_type d = size_type(-2);
-    static short_type r = short_type(-2);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(pfem, pf, 0);
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(size_type, d, size_type(-2));
+    DEFINE_STATIC_THREAD_LOCAL_INITIALIZED(short_type, r, short_type(-2));
     if (d != n || r != k) {
       std::stringstream name;
       name << "FEM_PK_PRISM(" << n << "," << k << ")";
